<?php 
@ini_set('memory_limit', '128M');

class thumbnailcreator {
    private $x, $y, $bild;
    private $width, $height, $typ;
    private $file, $fldr;
    
    /**
     * Inizalisiert das Objekt
     * @param string $file		Name der Bilddatei
     * @param string $fldr		Verzeichnis der Datei
     */
    public function __construct($file, $fldr) {
        $this->file = $file;
        $this->fldr = $fldr;
        $this->bild = NULL;
    }
    
    public function __destruct() {
        if (isset($this->bild)) {
            if (!@imagedestroy($this->bild)) {
                utilities::throwEX(__METHOD__, 'Keine gueltige Bilddatei: '.$this->file);
            }
        }
    }
    /**
     * setzt ein anderen Bildnamen
     * @param string $image
     */
    public function setFile($image) {
        if (isset($this->bild)) {
            if (!@imagedestroy($this->bild)) {
                utilities::throwEX(__METHOD__, 'Keine gueltige Bilddatei: '.$this->file);
            }
            $this->bild = NULL;
            $this->file = $image;
        }
    }
    /**
     * setzt ein anderes Verzeichnis indem sich Bilder befinden
     * @param string $fldr
     */
    public function setFldr($fldr) {
        $this->fldr = $fldr;
    }
    /**
     * liest ein Bild aus
     * und ermittelt so die Groesse des Bildes
     */
    public function readImage() {
        $this->setTyp();
        $this->bild = NULL;
        $file = $this->file;
        if (strlen($this->fldr) > 0) {
            $file = $this->fldr.'/'.$this->file;
        }
        switch ($this->typ) {
            case 'jpg':            
                if(!$this->bild = @imagecreatefromjpeg($file)){
                	@unlink($file);
					echo($this->file  . ' hatte Fehler und wurde geloescht');
					die();	
                }
                break;
            case 'gif':
                $this->bild = @imagecreatefromgif($file);
                break;
            case 'png':
                $this->bild = @imagecreatefrompng($file);
                break;
            case 'bmp':
                $this->bild = @imagewbmp($file);
                break;
            default:
                //unlink($this->fldr . '/' . $this->file);
                utilities::throwEX(__METHOD__, ': Keine gueltige Bilddatei die Datei');
                
        }
        if (!$this->bild) {
            utilities::throwEX(__METHOD__, ': Bild konnte nicht gelesen werden!');
        }
        if ($this->bild != NULL) {
            $this->x = imagesx($this->bild); // original width
            $this->y = imagesy($this->bild); // original height
            $this->setWidth();
            $this->setHeight();
        } else {
            utilities::throwEX(__METHOD__, ': Bild konnte nicht gelesen werden!');
        }
    }
    /**
     * gibt die Hoehe des ausgelesenen Images zurueck
     * @return integer $height
     */
    public function getHeight() {
        if (is_null($this->bild))
            throw new Exception(__METHOD__.' Fehler: '.$this->file.' wurde nicht ausgelesen!!');
        return $this->height;
    }
    /**
     * gibt die Breite des ausgelesenen Bildes zurueck
     * @return integer $width
     */
    public function getWidth() {
        if (is_null($this->bild))
            throw new Exception(__METHOD__.' Fehler: '.$this->file.' wurde nicht ausgelesen!!');
        return $this->width;
    }
    /**
     * gibt den Type des ausgelesenen Bildes zurueck
     * @return string $typ
     */
    public function getTyp() {
        if (! empty($this->typ)) {
            return $this->typ;
        } else {
            throw new Exception('Bildtyp wurde nicht bestimmt!');
        }
        
    }
    /**
     * verkleinert eine ausgelesene Bilddatei
     * auf eine vorgegebene Groesse und
     * speichert sie im gleichen Verzeichnis unter dem
     * uebergebenen Dateinamen wenn dieser angebenen wurde
     * ansonsten wird das Image als Bild ausgegeben
     * @param  string 		$resizedname => wenn kein Name uebergeben, wird das Image nicht gespeichert
     * @param  integer	$size
     */
    public function resizeImage($resizedname = 0, $size = 1) {
        $resizedwidth = ceil($this->width * $size);
        $resizedheight = ceil($this->height * $size);
        $resizedimage = imagecreatetruecolor($resizedwidth, $resizedheight);
        imagecopyresized($resizedimage, $this->bild, 0, 0, 0, 0, $resizedwidth, $resizedheight, $this->x, $this->y);
        $resized_filename = $resizedname;
        if (strlen($this->fldr) > 0) {
            $resized_filename = $this->fldr.'/'.$resizedname;
        }
        //myecho($resized_filename);
        if (is_numeric($resizedname)) {
            header('Content-type:image/jpeg');
            imagejpeg($resizedimage, $resized_filename);
            echo(file_get_contents($resized_filename));
        } else {
            imagejpeg($resizedimage, $resized_filename);
        }
        imagedestroy($resizedimage);
    }
    
    private function setHeight() {
        if (is_null($this->bild)) {
            throw new Exception(__METHOD__.': '.$this->file.' wurde nicht ausgelesen');
        }
        if ($this->y > 0) {
            $this->height = $this->y;
        } else {
            throw new Exception(__METHOD__.': Die Bildhoehe ist nicht gesetzt');
        }
        
    }
    
    private function setWidth() {
        if (is_null($this->bild)) {
            throw new Exception(__METHOD__.' Fehler: '.$this->file.' wurde nicht ausgelesen!!');
        }
        if ($this->x > 0) {
            $this->width = $this->x;
        } else {
            throw new Exception(__METHOD__.': Die Bildweite ist nicht gesetzt!');
        }
    }
    /**
     * ermittelt den ImageType
     */
    private function setTyp() {
        $file = $this->file;
        if (strlen($this->fldr) > 0) {
            $file = $this->fldr.'/'.$this->file;
        }
        if (is_file($file)) {
            // Bildtyp ermitteln
            $this->typ = strtolower(substr($file, strripos($file, '.') + 1));
        } else {
            throw new Exception(__METHOD__.": {$file} ist nicht vorhanden");
        }
    }
    /**
     * schneidet ein Bild
     * @return
     * @param integer $new_width
     * @param integer $new_height
     * @param string $destination_image
     */
    public function createThumbnail($new_width, $new_height, $destination_image) {
        global $CONFIG;
        $docroot = DOCROOT;
        $picture_realpath = $docroot.$CONFIG['IMAGEDIR'];
        
        if (!is_dir($picture_realpath)) {
            throw new Exception($picture_realpath.' ist nicht vorhanden');
        }
        if (!is_file($this->file)) {
            throw new Exception('OriginalBild wurde nicht gefunden');
        }
        $horizontal = $cropped = false;
        $image_id = substr($this->file, 0, strpos($this->file, '.'));
        if (!is_numeric($image_id)) {
            throw new Exception('Keine BildId');
        }
        if (!is_file($destination_image)) {
            /** seitenverhaeltnis abfragen */ if ($this->x > $this->y) {
                $horizontal = true;
            }
            // bild evtl. zuschneiden
            if ($horizontal) {
                if ($this->x > $new_width) {
                    $size = $new_width / $this->x;
                    $width = ceil($this->x * $size);
                    $height = ceil($this->y * $size);
                    if ($height < $new_height) {
                        $size = $new_height / $this->y;
                    }
                    $width = ceil($this->x * $size);
                    $height = ceil($this->y * $size);
                    $resized_file = $image_id."_{$width}x{$height}.temp.jpg";
                    self::resizeImage($resized_file, $size);
                    /** verkleinertes Bild als file setzen und auslesen */self::setFile($resized_file);
                    self::readImage();
                    $cut = ceil(($this->x - $new_width) / 2);
                    $direction = 'horizontal';
                }
            } else {
                if ($this->y > $new_height) {
                    $size = $new_height / $this->y;
                    $width = ceil($this->x * $size);
                    if ($width < $new_width) {
                        $size = $new_width / $this->x;
                    }
                    $width = ceil($this->x * $size);
                    $height = ceil($this->y * $size);
                    $resized_file = $image_id."_{$width}x{$height}.temp.jpg";
                    self::resizeImage($resized_file, $size);
                    /** verkleinertes Bild als file setzen und auslesen */self::setFile($resized_file);
                    self::readImage();
                    $cut = ceil(($this->y - $new_height) / 2);
                    $direction = 'vertical';
                }
            }
            self::crop($destination_image, $direction, $cut, $new_width, $new_height);
            /** verkleinertes Bild loeschen */unlink($resized_file);
        }
    }

    
    /**
     * schneidet einen Teil aus einem Bild
     * @return
     * @param object $file
     * @param object $direction
     * @param object $cut
     * @param object $width
     * @param object $height
     */
    public function crop($file, $direction, $cut, $width, $height) {
        global $CONFIG;
        if (strpos($file, '/') !== false) {
            $directory = DOCROOT.$CONFIG['IMAGEDIR'].substr($file, 0, strripos($file, '/') + 1);
            // myecho($directory);
            // myecho($file);
            // die();
            if (!is_dir($directory)) {
                throw new Exception($directory.' bitte mit Schreibrechten (je nach Servereinstellung 755 oder 777) anlegen');
            } else {
                $perm = substr(sprintf('%o', fileperms($directory)), -4);
				$perm_error = false;
                if ($perm !== '0777'){
                	$perm_error = true;
                }
				if($perm_error && $perm == '0755') {
					$perm_error = false;
                }
				if($perm_error){
					throw new Exception($directory.' bitte Schreibrechte (je nach Servereinstellung 755 oder 777) geben');
				}
            }
        }
        
        $cropped_image = imagecreatetruecolor($width, $height);
        /** teil aus dem Bild schneiden */ if ($direction == 'vertical') {
            imagecopyresized($cropped_image, $this->bild, 0, 0, 0, $cut, $width, $height, $width, $height);
        } else {
            imagecopyresized($cropped_image, $this->bild, 0, 0, $cut, 0, $width, $height, $width, $height);
        }
        $ext = substr($file, strripos($file, '.') + 1);
        switch (strtolower($ext)) {
            case 'jpg':
            case 'jpeg':
                imagejpeg($cropped_image, $file, 85);
                break;
            case 'gif':
                imagegif($cropped_image, $file);
                break;
            case 'png':
                imagepng($cropped_image, $file);
                break;
            case 'bmp':
                self::imagebmp($cropped_image, $file);
                break;
            default:
                throw new Exception('Bildtype croped ??');
        }
        
        imagedestroy($cropped_image);
    }
    
    private function imagebmp($im, $filename = '', $bit = 24, $compression = 0) {
        // version 1.00
        if (!in_array($bit, array(1, 4, 8, 16, 24, 32))) {
            $bit = 24;
        } else if ($bit == 32) {
            $bit = 24;
        }
        $bits = pow(2, $bit);
        imagetruecolortopalette($im, true, $bits);
        $width = imagesx($im);
        $height = imagesy($im);
        $colors_num = imagecolorstotal($im);
        $rgb_quad = '';
        if ($bit <= 8) {
            for ($i = 0; $i < $colors_num; $i++) {
                $colors = imagecolorsforindex($im, $i);
                $rgb_quad .= chr($colors['blue']).chr($colors['green']).chr($colors['red'])."\0";
            }
            $bmp_data = '';
            if ($compression == 0 || $bit < 8) {
                $compression = 0;
                $extra = '';
                $padding = 4 - ceil($width / (8 / $bit)) % 4;
                if ($padding % 4 != 0) {
                    $extra = str_repeat("\0", $padding);
                }
                for ($j = $height - 1; $j >= 0; $j--) {
                    $i = 0;
                    while ($i < $width) {
                        $bin = 0;
                        $limit = $width - $i < 8 / $bit ? (8 / $bit - $width + $i) * $bit : 0;
                        for ($k = 8 - $bit; $k >= $limit; $k -= $bit) {
                            $index = imagecolorat($im, $i, $j);
                            $bin |= $index << $k;
                            $i++;
                        }
                        $bmp_data .= chr($bin);
                    }
                    $bmp_data .= $extra;
                }
            }
            // RLE8
            else if ($compression == 1 && $bit == 8) {
                for ($j = $height - 1; $j >= 0; $j--) {
                    $last_index = "\0";
                    $same_num = 0;
                    for ($i = 0; $i <= $width; $i++) {
                        $index = imagecolorat($im, $i, $j);
                        if ($index !== $last_index || $same_num > 255) {
                            if ($same_num != 0) {
                                $bmp_data .= chr($same_num).chr($last_index);
                            }
                            $last_index = $index;
                            $same_num = 1;
                        } else {
                            $same_num++;
                        }
                    }
                    $bmp_data .= "\0\0";
                }
                $bmp_data .= "\0\1";
            }
            $size_quad = strlen($rgb_quad);
            $size_data = strlen($bmp_data);
        } else {
            $extra = '';
            $padding = 4 - ($width * ($bit / 8)) % 4;
            if ($padding % 4 != 0) {
                $extra = str_repeat("\0", $padding);
            }
            $bmp_data = '';
            for ($j = $height - 1; $j >= 0; $j--) {
                for ($i = 0; $i < $width; $i++) {
                    $index = imagecolorat($im, $i, $j);
                    $colors = imagecolorsforindex($im, $index);
                    if ($bit == 16) {
                        $bin = 0 << $bit;
                        $bin |= ($colors['red'] >> 3) << 10;
                        $bin |= ($colors['green'] >> 3) << 5;
                        $bin |= $colors['blue'] >> 3;
                        $bmp_data .= pack("v", $bin);
                    } else {
                        $bmp_data .= pack("c*", $colors['blue'], $colors['green'], $colors['red']);
                    }
                }
                $bmp_data .= $extra;
            }
            $size_quad = 0;
            $size_data = strlen($bmp_data);
            $colors_num = 0;
        }
        $file_header = 'BM'.pack('V3', 54 + $size_quad + $size_data, 0, 54 + $size_quad);
        $info_header = pack('V3v2V*', 0x28, $width, $height, 1, $bit, $compression, $size_data, 0, 0, $colors_num, 0);
        if ($filename != '') {
            $fp = fopen($filename, 'wb');
            fwrite($fp, $file_header.$info_header.$rgb_quad.$bmp_data);
            fclose($fp);
            return true;
        }
        //echo $file_header . $info_header. $rgb_quad . $bmp_data;
        return true;
    }
    
    /**
     * erstellt den Header fuer die
     * Ausgabe des Bildes entsprechend des
     * Imagetypes
     */
    public function createHeader() {
    	header("HTTP/1.0 200 OK"); 
        if (! empty($this->typ)) {
            switch (strtolower($this->typ)) {
                case 'jpg':
                case 'jpeg':
                    header('Content-type:image/jpeg');
                    break;
                case 'gif':
                case 'png':
                case 'bmp':
                    header('Content-type:image/'.strtolower($this->typ));
                    break;
                default:
                    throw new Exception($this->type.' kann kein Header gesetzt werden!!');
            }
        } else {
            throw new Exception('Erst den ImageType ermitteln!!');
        }
    }
}
?>
