<?php 
class utilities {
    /**
     * Uberprueft ob eine Datei auf einem anderen Server vorhanden ist
     * und der Server erreichbar ist
     * @param string $host
     * @param string $loc
     *
     * @return boolean   true bei vorhanden / false bei nicht vorhanden oder nicht erreichbar
     */
    static function head($host, $loc) {
        $errno = $status = 0;
        $errstr = "$loc not found";
        try {
            $fp = fsockopen($host, 80, $errno, $errst, 15);
            if ($fp) {
                fputs($fp, "HEAD $loc HTTP/1.0\r\n");
                fputs($fp, "Host: $host\r\n");
                fputs($fp, "\r\n");
                while (!feof($fp)) {
                    $c = fgets($fp, 128);
                    if (strpos("~".$c, "HTTP") == 1)
                        $status = 0 + substr($c, strpos($c, " ") + 1, 3);
                }
                fclose($fp);
                if ($status != 200) {
                    throw new Exception(__METHOD__.": Error! Status: $status Errno: $errno Errstr: $errstr");
                }
                return true; // 200 =OK;  404 =not Found etc.
            } else {
                throw new Exception(__METHOD__.": No Connection to Host $host");
            }
        }
        catch(Exception $ex) {
            $message = $ex->getMessage();
            self::savelog($message);
            return false;
        }
        return true;
    }
    /**
     * gibt eine Fehlermeldung zurueck
     * auf der Entwicklung wird die Klasse::Funktion
     * mit dargestellt
     *
     * @param objct 	$method
     * @param string 	$error
     */
    static function throwEx($method, $error) {
        // log error
        self::savelog($error);
        throw new Exception($error);
    }
    /**
     * erstellt einen neuen Eintrag id die log.txt
     * @param string $text
     * @return boolean
     */
    static function savelog($text) {
        global $CONFIG;
        if (!is_array($CONFIG)) {
            return false;
        }
        if (isset($CONFIG['DEVELOPMENT']) && $CONFIG['DEVELOPMENT']) {
            $CONFIG['LOG_TEXT'] .= '<div style="padding:2px;text-align:left;background-color:#000;color:#FFF;font-size:12px;font-family:verdana,arial;margin:0 auto;width:900px;">'.$text.'</div>';
        }
        if ($fp = @fopen($CONFIG['TEMPDIR']."log.txt", "a")) {
            fputs($fp, date("H:i:s d.m.Y")." - ".__FILE__." - ".$text."\n");
            fclose($fp);
            @chmod($CONFIG['TEMPDIR']."log.txt", 0666);
        }
        return true;
    }
    /**
     * liest eine Datei aus und gibt den Inhalt zurueck
     * @param string $remote_file
     * @return string
     * bei Fehler boolean
     */
    static function getContentByCurl($remote_file) {
        $content = null;
        $cURL = curl_init();
        curl_setopt($cURL, CURLOPT_URL, $remote_file);
        curl_setopt($cURL, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($cURL, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($cURL, CURLOPT_HEADER, 1);
        $results = curl_exec($cURL);
        if ($results) {
            $code = curl_getinfo($cURL, CURLINFO_HTTP_CODE);
            $header_size = curl_getinfo($cURL, CURLINFO_HEADER_SIZE);
            curl_close($cURL);
            $tempheaders = substr($results, 0, $header_size);
            $content = substr($results, $header_size);
            $tempheaders = explode("\n", $tempheaders);
            //myecho($tempheaders);
            if (strpos($tempheaders[0], '200') === false) {
                savelog($remote_file.' wurde nicht gefunden');
                return false;
            }
        } else {
            savelog($remote_file.' ist unbekannt');
            return false;
        }
        return $content;
    }
    
    /**
     * Ueberprueft ob ein String im UTF-8 Format uebergeben wurde
     * wenn ja wird true zurueck gegeben, sonst false
     * @param string $str
     * @return boolean
     */
    public static function isUTF8($string) {
        //return (mb_detect_encoding($string) === 'UTF-8' || utf8_encode(utf8_decode($string)) == $string);
        return (utf8_encode(utf8_decode($string)) == $string);
    }

    
    /**
     * Encodes String to UTF8
     * konvertiert nur nach UTF8, wenn der String nicht
     * schon in UFT8 kodiert ist
     * @param string $string
     * @return string
     */
    public static function __utf8_encode($string) {
        $bResult = $string;
        if (is_array($string)) {
            $bResult = array();
            foreach ($string as $key=>$value) {
                if (!self::isUTF8($string)) {
                    $value = utf8_encode($value);
                }
                $bResult[$key] = $value;
            }
        } else {
            if (!self::isUTF8($string)) {
                $bResult = utf8_encode($string);
            }
        }
        return $bResult;
    }
    
    /**
     * Decodes String from UTF8
     * konvertiert von UTF8 nach ISO, wenn der String nicht
     * schon in UFT8 kodiert ist
     * @param string $string
     * @return string
     */
    public static function __utf8_decode($string) {
        $bResult = $string;
        if (is_array($string)) {
            $bResult = array();
            foreach ($string as $key=>$value) {
                if (self::isUTF8($string)) {
                    $value = utf8_decode($value);
                }
                $bResult[$key] = $value;
            }
        } else {
            if (self::isUTF8($string)) {
                $bResult = utf8_decode($string);
            }
        }
        return $bResult;
    }
    
    /**
     * konvertiert einen String
     * aus der Datenbank oder XML
     * zu ISO-8859-1
     * @param string $string
     *
     * @return string
     */
    static function fixText($string, $entities = false) {
        $string = self::__utf8_decode($string);
        if ($entities) {
            $string = htmlentities($string);
        }
        return $string;
    }
    
    /**
     * converts string to utf8 if it wasn't
     * @param string $text
     */
    public static function fixUTF8($string) {
        return $string;
        $bResult = null;
        if (is_array($text)) {
            $bResult = array();
            foreach ($string as $key=>$value) {
                $enc = self::getEncoding($value);
                $value = (self::isUTF8($value)) ? (string) $value : iconv($enc, "UTF-8", (string) $value);
                $bResult[$key] = $value;
            }
        } else {
            $enc = self::getEncoding($string);
            $bResult = (self::isUTF8($string)) ? (string) $string : iconv($enc, "UTF-8", (string) $string);
        }
        return $bResult;
    }
    
    /**
     * list ein Vezeichnis rekursiv aus
     * und gibt die gefundenen Dateienamen
     * ine einer Liste zurueck
     * @param string $dir
     * @param array $list
     * @return array %filelist
     */
    public static function dir_recursive($dir, $filelist) {
        $handle = opendir($dir);
        while ($file = readdir($handle)) {
            if ($file != "." && $file != "..") {
                // Wenn Verzeichniseintrag ein Verzeichnis ist
                if (is_dir($dir.$file)) {
                    // Erneuter Funktionsaufruf, um das aktuelle Verzeichnis auszulesen
                    $filelist = self::dir_recursive($dir.$file.'/', $filelist);
                } else {
                    // Wenn Verzeichnis-Eintrag eine Datei ist, diese ausgeben
                    array_push($filelist, $dir.$file);
                }
                
            }
            
        }
        closedir($handle);
        return $filelist;
    }
    
    public static function getEncoding($text) {
    
        $bResult = 'UTF-8';
        $encodings = "UTF-8, ISO-8859-1, ISO-8859-15, cp1252";
        $encoding = mb_detect_encoding($text, $encodings, true);
        if ($encoding != 'UTF-8') {
            $bResult = $encoding;
        }
        return $bResult;
        
    }
    
    public static function convert_memory_usage() {
        $bResult = '';
        $mem_usage = memory_get_usage(true);
        if ($mem_usage < 1024) {
            $bResult = $mem_usage." bytes";
        } elseif ($mem_usage < 1048576) {
            $bResult = round($mem_usage / 1024, 2)." kilobytes";
        } else {
            $bResult = round($mem_usage / 1048576, 2)." megabytes";
        }
        return $bResult;
    }
    
    /**
     * gibt den Verzeichnispfad je nach Servereinstellung
     * mit / am Ende aus
     * @return string
     */
    public static function getDocRoot() {
        $docroot = realpath(dirname(__file__));
        $docroot = str_ireplace(array("\\", "_system"), array("/", ""), $docroot);
        return $docroot;
    }
    /**
     * gibt den HTTP-Pfad zum Verzeichnis des 777Script zurueck
     * z. B. bei Installation im Webroot 	=> /
     * oder als Unterverzeichnis 			=> scriptverzeichnis/
     *
     * @param object $file
     * @param string $sub_directory
     * @return string
     */
    public static function getScriptDirectory($file, $sub_directory) {
        if (substr($sub_directory, strlen($sub_directory) - 1) != '/') {
            $sub_directory .= '/';
        }
        $docroot = utilities::getDocRoot();
        $bResult = str_ireplace(utilities::getDocRoot(), '/', $docroot);
		$bResult = $docroot;
        $bResult = str_ireplace(array('\\', $sub_directory), array('/', ''), $bResult);
        // myecho($bResult);
        $bResult = str_ireplace(utilities::getDocRoot(), '/', $bResult);
        if (strlen($bResult) == 0) {
            $bResult = '/';
        }
        if (substr($bResult, strlen($bResult) - 1) != '/') {
            $bResult .= '/';
        }
        // myecho($bResult);die();
        return $bResult;
    }
    
}
?>
